@@ -18,7 +18,7 @@ from supervised.utils.config import LOG_LEVEL
 logger = logging.getLogger(__name__)
 logger.setLevel(LOG_LEVEL)
 
-from catboost import CatBoostClassifier, CatBoostRegressor, CatBoost
+from catboost import CatBoostClassifier, CatBoostRegressor, CatBoost, Pool
 import catboost
 
 
@@ -92,7 +92,16 @@ class CatBoostAlgorithm(BaseAlgorithm):
         except Exception as e:
             return 1000
 
-    def fit(self, X, y, X_validation=None, y_validation=None, log_to_file=None):
+    def fit(
+        self,
+        X,
+        y,
+        sample_weight=None,
+        X_validation=None,
+        y_validation=None,
+        sample_weight_validation=None,
+        log_to_file=None,
+    ):
         if self.model.tree_count_ is not None:
             print("CatBoost model already fitted. Skip fit().")
             return
@@ -105,7 +114,12 @@ class CatBoostAlgorithm(BaseAlgorithm):
 
         eval_set = None
         if X_validation is not None and y_validation is not None:
-            eval_set = (X_validation, y_validation)
+            eval_set = Pool(
+                data=X_validation,
+                label=y_validation,
+                cat_features=self.cat_features,
+                weight=sample_weight_validation,
+            )
 
         new_iterations = self._assess_iterations(X, y, eval_set)
         self.model.set_params(iterations=new_iterations)
@@ -113,6 +127,7 @@ class CatBoostAlgorithm(BaseAlgorithm):
         self.model.fit(
             X,
             y,
+            sample_weight=sample_weight,
             cat_features=self.cat_features,
             init_model=None if self.model.tree_count_ is None else self.model,
             eval_set=eval_set,
@@ -181,10 +196,22 @@ class CatBoostAlgorithm(BaseAlgorithm):
     def file_extension(self):
         return "catboost"
 
+    def get_metric_name(self):
+        metric = self.params.get("loss_function")
+        if metric is None:
+            return None
+        if metric == "Logloss":
+            return "logloss"
+        elif metric == "MultiClass":
+            return "logloss"
+        elif metric == "RMSE":
+            return "rmse"
+        return None
+
 
 classification_params = {
     "learning_rate": [0.05, 0.1, 0.2],
-    "depth": [2, 3, 4, 5, 6],
+    "depth": [4, 5, 6, 7, 8, 9],
     "rsm": [0.7, 0.8, 0.9, 1],  # random subspace method
     "loss_function": ["Logloss"],
 }
@@ -192,7 +219,7 @@ classification_params = {
 classification_default_params = {
     "learning_rate": 0.1,
     "depth": 6,
-    "rsm": 0.9,
+    "rsm": 1,
     "loss_function": "Logloss",
 }
 
@@ -221,9 +248,14 @@ AlgorithmsRegistry.add(
 
 multiclass_classification_params = copy.deepcopy(classification_params)
 multiclass_classification_params["loss_function"] = ["MultiClass"]
+multiclass_classification_params["depth"] = [3, 4, 5, 6]
+multiclass_classification_params["learning_rate"] = [0.1, 0.2]
 
 multiclass_classification_default_params = copy.deepcopy(classification_default_params)
 multiclass_classification_default_params["loss_function"] = "MultiClass"
+multiclass_classification_default_params["depth"] = 5
+multiclass_classification_default_params["learning_rate"] = 0.1
+
 
 AlgorithmsRegistry.add(
     MULTICLASS_CLASSIFICATION,
@@ -235,7 +267,7 @@ AlgorithmsRegistry.add(
 )
 
 regression_params = copy.deepcopy(classification_params)
-regression_params["loss_function"] = ["RMSE"]
+regression_params["loss_function"] = ["RMSE", "MAE"]
 
 regression_required_preprocessing = [
     "missing_values_inputation",
@@ -248,7 +280,7 @@ regression_required_preprocessing = [
 regression_default_params = {
     "learning_rate": 0.1,
     "depth": 6,
-    "rsm": 0.9,
+    "rsm": 1,
     "loss_function": "RMSE",
 }
 
