@@ -58,9 +58,19 @@ class LightgbmAlgorithm(BaseAlgorithm):
     def update(self, update_params):
         pass
 
-    def fit(self, X, y, X_validation=None, y_validation=None, log_to_file=None):
-
-        lgb_train = lgb.Dataset(X.to_numpy() if isinstance(X, pd.DataFrame) else X, y)
+    def fit(
+        self,
+        X,
+        y,
+        sample_weight=None,
+        X_validation=None,
+        y_validation=None,
+        sample_weight_validation=None,
+        log_to_file=None,
+    ):
+        lgb_train = lgb.Dataset(
+            X.to_numpy() if isinstance(X, pd.DataFrame) else X, y, weight=sample_weight
+        )
         if self.early_stopping_rounds == 0:
             self.model = lgb.train(
                 self.learner_params,
@@ -80,6 +90,7 @@ class LightgbmAlgorithm(BaseAlgorithm):
                         if isinstance(X_validation, pd.DataFrame)
                         else X_validation,
                         y_validation,
+                        weight=sample_weight_validation,
                     ),
                 ]
                 valid_names = ["train", "validation"]
@@ -145,22 +156,36 @@ class LightgbmAlgorithm(BaseAlgorithm):
         self.uid = json_desc.get("uid", self.uid)
         self.params = json_desc.get("params", self.params)
 
+    def get_metric_name(self):
+        metric = self.params.get("metric")
+        if metric is None:
+            return None
+        if metric == "binary_logloss":
+            return "logloss"
+        elif metric == "auc":
+            return "auc"
+        elif metric == "multi_logloss":
+            return "logloss"
+        elif metric == "rmse":
+            return "rmse"
+        return None
+
 
 lgbm_bin_params = {
     "objective": ["binary"],
     "metric": ["binary_logloss", "auc"],
-    "num_leaves": [3, 7, 15, 31],  # , 63, 127],
-    "learning_rate": [0.05, 0.075, 0.1, 0.15],
-    "feature_fraction": [0.8, 0.9, 1.0],
-    "bagging_fraction": [0.8, 0.9, 1.0],
+    "num_leaves": [15, 31, 63, 95, 127],
+    "learning_rate": [0.05, 0.1, 0.2],
+    "feature_fraction": [0.5, 0.8, 0.9, 1.0],
+    "bagging_fraction": [0.5, 0.8, 0.9, 1.0],
     "min_data_in_leaf": [5, 10, 15, 20, 30, 50],
 }
 
 classification_bin_default_params = {
     "objective": "binary",
     "metric": "binary_logloss",
-    "num_leaves": 31,
-    "learning_rate": 0.1,
+    "num_leaves": 63,
+    "learning_rate": 0.05,
     "feature_fraction": 0.9,
     "bagging_fraction": 0.9,
     "min_data_in_leaf": 10,
@@ -189,8 +214,8 @@ lgbm_multi_params["metric"] = ["multi_logloss"]
 classification_multi_default_params = {
     "objective": "multiclass",
     "metric": "multi_logloss",
-    "num_leaves": 31,
-    "learning_rate": 0.1,
+    "num_leaves": 63,
+    "learning_rate": 0.05,
     "feature_fraction": 0.9,
     "bagging_fraction": 0.9,
     "min_data_in_leaf": 10,
@@ -230,8 +255,8 @@ regression_required_preprocessing = [
 regression_default_params = {
     "objective": "regression",
     "metric": "rmse",
-    "num_leaves": 15,
-    "learning_rate": 0.1,
+    "num_leaves": 63,
+    "learning_rate": 0.05,
     "feature_fraction": 0.9,
     "bagging_fraction": 0.9,
     "min_data_in_leaf": 10,
