@@ -71,9 +71,21 @@ class XgbAlgorithm(BaseAlgorithm):
         self.best_ntree_limit = 0
         logger.debug("XgbLearner __init__")
 
-    def fit(self, X, y, X_validation=None, y_validation=None, log_to_file=None):
+    def fit(
+        self,
+        X,
+        y,
+        sample_weight=None,
+        X_validation=None,
+        y_validation=None,
+        sample_weight_validation=None,
+        log_to_file=None,
+    ):
         dtrain = xgb.DMatrix(
-            X.to_numpy() if isinstance(X, pd.DataFrame) else X, label=y, missing=np.NaN
+            X.to_numpy() if isinstance(X, pd.DataFrame) else X,
+            label=y,
+            missing=np.NaN,
+            weight=sample_weight,
         )
         dvalidation = xgb.DMatrix(
             X_validation.to_numpy()
@@ -81,6 +93,7 @@ class XgbAlgorithm(BaseAlgorithm):
             else X_validation,
             label=y_validation,
             missing=np.NaN,
+            weight=sample_weight_validation,
         )
         evals_result = {}
 
@@ -175,22 +188,36 @@ class XgbAlgorithm(BaseAlgorithm):
     def file_extension(self):
         return "xgboost"
 
+    def get_metric_name(self):
+        metric = self.params.get("eval_metric")
+        if metric is None:
+            return None
+        if metric == "logloss":
+            return "logloss"
+        elif metric == "auc":
+            return "auc"
+        elif metric == "mlogloss":
+            return "logloss"
+        elif metric == "rmse":
+            return "rmse"
+        return None
+
 
 # For binary classification target should be 0, 1. There should be no NaNs in target.
 xgb_bin_class_params = {
     "objective": ["binary:logistic"],
     "eval_metric": ["logloss", "auc"],
     "eta": [0.05, 0.075, 0.1, 0.15],
-    "max_depth": [1, 2, 3, 4, 5, 6, 7, 8, 9],
-    "min_child_weight": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
-    "subsample": [0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0],
-    "colsample_bytree": [0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0],
+    "max_depth": [4, 5, 6, 7, 8, 9],
+    "min_child_weight": [1, 5, 10, 25, 50],
+    "subsample": [0.5, 0.6, 0.7, 0.8, 0.9, 1.0],
+    "colsample_bytree": [0.5, 0.6, 0.7, 0.8, 0.9, 1.0],
 }
 
 classification_bin_default_params = {
     "objective": "binary:logistic",
     "eval_metric": "logloss",
-    "eta": 0.1,
+    "eta": 0.075,
     "max_depth": 6,
     "min_child_weight": 1,
     "subsample": 1.0,
@@ -199,8 +226,8 @@ classification_bin_default_params = {
 
 xgb_regression_params = dict(xgb_bin_class_params)
 xgb_regression_params["objective"] = ["reg:squarederror"]
-xgb_regression_params["eval_metric"] = ["rmse"]
-xgb_regression_params["max_depth"] = [1, 2, 3, 4]
+xgb_regression_params["eval_metric"] = ["rmse", "mae"]
+xgb_regression_params["max_depth"] = [4, 5, 6, 7, 8, 9]
 
 
 xgb_multi_class_params = dict(xgb_bin_class_params)
@@ -210,7 +237,7 @@ xgb_multi_class_params["eval_metric"] = ["mlogloss"]
 classification_multi_default_params = {
     "objective": "multi:softprob",
     "eval_metric": "mlogloss",
-    "eta": 0.1,
+    "eta": 0.075,
     "max_depth": 6,
     "min_child_weight": 1,
     "subsample": 1.0,
@@ -221,8 +248,8 @@ classification_multi_default_params = {
 regression_default_params = {
     "objective": "reg:squarederror",
     "eval_metric": "rmse",
-    "eta": 0.1,
-    "max_depth": 4,
+    "eta": 0.075,
+    "max_depth": 6,
     "min_child_weight": 1,
     "subsample": 1.0,
     "colsample_bytree": 1.0,
