@@ -30,7 +30,6 @@ from supervised.model_framework import ModelFramework
 from supervised.preprocessing.exclude_missing_target import ExcludeRowsMissingTarget
 from supervised.tuner.data_info import DataInfo
 from supervised.tuner.mljar_tuner import MljarTuner
-from supervised.utils.additional_metrics import AdditionalMetrics
 from supervised.utils.config import mem
 from supervised.utils.config import LOG_LEVEL
 from supervised.utils.leaderboard_plots import LeaderboardPlots
@@ -92,6 +91,8 @@ class BaseAutoML(BaseEstimator, ABC):
         # https://scikit-learn.org/stable/developers/develop.html#universal-attributes
         self.n_features_in_ = None  # for scikit-learn api
         self.tuner = None
+        self._boost_on_errors = None
+        self._kmeans_features = None
 
     def _get_tuner_params(
         self, start_random_models, hill_climbing_steps, top_models_to_improve
@@ -149,6 +150,8 @@ class BaseAutoML(BaseEstimator, ABC):
             self._top_models_to_improve = params.get(
                 "top_models_to_improve", self._top_models_to_improve
             )
+            self._boost_on_errors = params.get("boost_on_errors", self._boost_on_errors)
+            self._kmeans_features = params.get("kmeans_features", self._kmeans_features)
             self._random_state = params.get("random_state", self._random_state)
             stacked_models = params.get("stacked")
 
@@ -233,6 +236,9 @@ class BaseAutoML(BaseEstimator, ABC):
         self._time_ctrl.log_time(
             model.get_name(), model.get_type(), self._fit_level, model.get_train_time()
         )
+        
+        self.tuner.add_key(model)
+        
 
     def create_dir(self, model_path):
         if not os.path.exists(model_path):
@@ -325,8 +331,8 @@ class BaseAutoML(BaseEstimator, ABC):
             self.ensemble = Ensemble(
                 self._eval_metric, self._ml_task, is_stacked=is_stacked
             )
-            oofs, target = self.ensemble.get_oof_matrix(self._models)
-            self.ensemble.fit(oofs, target)
+            oofs, target, sample_weight = self.ensemble.get_oof_matrix(self._models)
+            self.ensemble.fit(oofs, target, sample_weight)
             self.ensemble.save(ensemble_path)
             self.keep_model(self.ensemble, ensemble_path)
             return True
@@ -452,18 +458,26 @@ class BaseAutoML(BaseEstimator, ABC):
             self.train_model(params)
         """
 
-    def _save_data(self, X, y):
+    def _save_data(self, X, y, sample_weight=None):
         # save information about original data
-        self._save_data_info(X, y)
+        self._save_data_info(X, y, sample_weight)
 
         # handle drastic imbalance
         # assure at least 20 samples of each class
         # for binary and multiclass classification
-        self._handle_drastic_imbalance(X, y)
+        self._handle_drastic_imbalance(X, y, sample_weight)
 
         # prepare path for saving files
         self._X_path = os.path.join(self._results_path, "X.parquet")
         self._y_path = os.path.join(self._results_path, "y.parquet")
+        self._sample_weight_path = None
+        if sample_weight is not None:
+            self._sample_weight_path = os.path.join(
+                self._results_path, "sample_weight.parquet"
+            )
+            pd.DataFrame({"sample_weight": sample_weight}).to_parquet(
+                self._sample_weight_path, index=False
+            )
 
         X.to_parquet(self._X_path, index=False)
 
@@ -476,10 +490,12 @@ class BaseAutoML(BaseEstimator, ABC):
         self._validation_strategy["X_path"] = self._X_path
         self._validation_strategy["y_path"] = self._y_path
         self._validation_strategy["results_path"] = self._results_path
+        if sample_weight is not None:
+            self._validation_strategy["sample_weight_path"] = self._sample_weight_path
 
         self._drop_data_variables(X)
 
-    def _handle_drastic_imbalance(self, X, y):
+    def _handle_drastic_imbalance(self, X, y, sample_weight=None):
         if self._ml_task == REGRESSION:
             return
         classes, cnts = np.unique(y, return_counts=True)
@@ -496,11 +512,21 @@ class BaseAutoML(BaseEstimator, ABC):
                     .sample(n=append_samples, replace=True, random_state=1)
                     .reset_index(drop=True)
                 )
+                if sample_weight is not None:
+                    new_sample_weight = (
+                        sample_weight[y == classes[i]]
+                        .sample(n=append_samples, replace=True, random_state=1)
+                        .reset_index(drop=True)
+                    )
                 for j in range(new_X.shape[0]):
                     X.loc[X.shape[0]] = new_X.loc[j]
                     y.loc[y.shape[0]] = classes[i]
+                    if sample_weight is not None:
+                        sample_weight.loc[
+                            sample_weight.shape[0]
+                        ] = new_sample_weight.loc[j]
 
-    def _save_data_info(self, X, y):
+    def _save_data_info(self, X, y, sample_weight=None):
 
         target_is_numeric = pd.api.types.is_numeric_dtype(y)
         if self._ml_task == MULTICLASS_CLASSIFICATION:
@@ -519,6 +545,7 @@ class BaseAutoML(BaseEstimator, ABC):
             "columns_info": columns_and_target_info["columns_info"],
             "target_info": columns_and_target_info["target_info"],
             "n_features": self.n_features_in_,
+            "is_sample_weighted": sample_weight is not None,
         }
         # Add n_classes if not regression
         if self._ml_task != REGRESSION:
@@ -581,7 +608,7 @@ class BaseAutoML(BaseEstimator, ABC):
 
     # This method builds pandas.Dataframe from input. The input can be numpy.ndarray, matrix, or pandas.Dataframe
     # This method is used to build dataframes in `fit()` and in `predict`. That's the reason y can be None (`predict()` method)
-    def _build_dataframe(self, X, y=None):
+    def _build_dataframe(self, X, y=None, sample_weight=None):
         if X is None or X.shape[0] == 0:
             raise AutoMLException("Empty input dataset")
         # If Inputs are not pandas dataframes use scikit-learn validation for X array
@@ -617,12 +644,26 @@ class BaseAutoML(BaseEstimator, ABC):
             y = check_array(y, ensure_2d=False)
             y = pd.Series(np.array(y), name="target")
 
-        X, y = ExcludeRowsMissingTarget.transform(X, y, warn=True)
+        if sample_weight is not None:
+            if isinstance(sample_weight, np.ndarray):
+                sample_weight = check_array(sample_weight, ensure_2d=False)
+                sample_weight = pd.Series(np.array(sample_weight), name="sample_weight")
+            elif isinstance(sample_weight, pd.DataFrame):
+                sample_weight = np.array(sample_weight.iloc[:, 0])
+                sample_weight = check_array(sample_weight, ensure_2d=False)
+                sample_weight = pd.Series(np.array(sample_weight), name="sample_weight")
+
+        X, y, sample_weight = ExcludeRowsMissingTarget.transform(
+            X, y, sample_weight, warn=True
+        )
 
         X.reset_index(drop=True, inplace=True)
         y.reset_index(drop=True, inplace=True)
 
-        return X, y
+        if sample_weight is not None:
+            sample_weight.reset_index(drop=True, inplace=True)
+
+        return X, y, sample_weight
 
     def _apply_constraints(self):
 
@@ -730,17 +771,31 @@ class BaseAutoML(BaseEstimator, ABC):
             if self._stack_models:
                 self.verbose_print("Disable stacking for split validation")
             self._stack_models = False
+            self._boost_on_errors = False
         if "repeats" in self._validation_strategy:
             if self._stack_models:
                 self.verbose_print("Disable stacking for repeated validation")
             self._stack_models = False
+            self._boost_on_errors = False
 
+        # update Tuner
         if self.tuner is not None:
             self.tuner._stack_models = self._stack_models
+            self.tuner._boost_on_errors = self._boost_on_errors
+
+        # update Time Controler
         if self._time_ctrl is not None:
             self._time_ctrl._is_stacking = self._stack_models
 
-    def _fit(self, X, y):
+            if "stack" in self._time_ctrl._steps and not self._stack_models:
+                self._time_ctrl._steps.remove("stack")
+            if (
+                "boost_on_errors" in self._time_ctrl._steps
+                and not self._boost_on_errors
+            ):
+                self._time_ctrl._steps.remove("boost_on_errors")
+
+    def _fit(self, X, y, sample_weight=None):
         """Fits the AutoML model with data"""
         if self._fit_level == "finished":
             print(
@@ -748,7 +803,7 @@ class BaseAutoML(BaseEstimator, ABC):
             )
             return
         # Validate input and build dataframes
-        X, y = self._build_dataframe(X, y)
+        X, y, sample_weight = self._build_dataframe(X, y, sample_weight)
 
         self.n_rows_in_ = X.shape[0]
         self.n_features_in_ = X.shape[1]
@@ -772,6 +827,8 @@ class BaseAutoML(BaseEstimator, ABC):
         self._start_random_models = self._get_start_random_models()
         self._hill_climbing_steps = self._get_hill_climbing_steps()
         self._top_models_to_improve = self._get_top_models_to_improve()
+        self._boost_on_errors = self._get_boost_on_errors()
+        self._kmeans_features = self._get_kmeans_features()
         self._random_state = self._get_random_state()
 
         self._adjust_validation = False
@@ -812,7 +869,7 @@ class BaseAutoML(BaseEstimator, ABC):
                 EDA.compute(X, y, os.path.join(self._results_path, "EDA"))
 
             # Save data
-            self._save_data(X.copy(deep=False), y)
+            self._save_data(X.copy(deep=False), y, sample_weight)
 
             tuner = MljarTuner(
                 self._get_tuner_params(
@@ -830,6 +887,8 @@ class BaseAutoML(BaseEstimator, ABC):
                 self._train_ensemble,
                 self._stack_models,
                 self._adjust_validation,
+                self._boost_on_errors,
+                self._kmeans_features,
                 self._random_state,
             )
             self.tuner = tuner
@@ -991,6 +1050,8 @@ class BaseAutoML(BaseEstimator, ABC):
                 "start_random_models": self._start_random_models,
                 "hill_climbing_steps": self._hill_climbing_steps,
                 "top_models_to_improve": self._top_models_to_improve,
+                "boost_on_errors": self._boost_on_errors,
+                "kmeans_features": self._kmeans_features,
                 "random_state": self._random_state,
                 "saved": self._model_paths,
             }
@@ -1003,11 +1064,11 @@ class BaseAutoML(BaseEstimator, ABC):
             ldb.to_csv(os.path.join(self._results_path, "leaderboard.csv"), index=False)
 
             # save report
-            ldb["Link"] = [f"[Results link]({m}/README.md)" for m in ldb["name"].values]
             ldb.insert(loc=0, column="Best model", value="")
             ldb.loc[
                 ldb.name == self._best_model.get_name(), "Best model"
             ] = "**the best**"
+            ldb["name"] = [f"[{m}]({m}/README.md)" for m in ldb["name"].values]
 
             with open(os.path.join(self._results_path, "README.md"), "w") as fout:
                 fout.write(f"# AutoML Leaderboard\n\n")
@@ -1112,7 +1173,7 @@ class BaseAutoML(BaseEstimator, ABC):
         # Make and return predictions
         return self._base_predict(X)
 
-    def _score(self, X, y=None):
+    def _score(self, X, y=None, sample_weight=None):
         # y default must be None for scikit-learn compatibility
 
         # Check if y is None
@@ -1121,9 +1182,9 @@ class BaseAutoML(BaseEstimator, ABC):
 
         predictions = self._predict(X)
         return (
-            r2_score(y, predictions)
+            r2_score(y, predictions, sample_weight=sample_weight)
             if self._ml_task == REGRESSION
-            else accuracy_score(y, predictions)
+            else accuracy_score(y, predictions, sample_weight=sample_weight)
         )
 
     def _get_mode(self):
@@ -1381,6 +1442,32 @@ class BaseAutoML(BaseEstimator, ABC):
         else:
             return deepcopy(self.top_models_to_improve)
 
+    def _get_boost_on_errors(self):
+        """ Gets the current boost_on_errors"""
+        self._validate_boost_on_errors()
+        if self.boost_on_errors == "auto":
+            if self._get_mode() == "Explain":
+                return False
+            if self._get_mode() == "Perform":
+                return False
+            if self._get_mode() == "Compete":
+                return True
+        else:
+            return deepcopy(self.boost_on_errors)
+
+    def _get_kmeans_features(self):
+        """ Gets the current kmeans_features"""
+        self._validate_kmeans_features()
+        if self.kmeans_features == "auto":
+            if self._get_mode() == "Explain":
+                return False
+            if self._get_mode() == "Perform":
+                return False
+            if self._get_mode() == "Compete":
+                return True
+        else:
+            return deepcopy(self.kmeans_features)
+
     def _get_random_state(self):
         """ Gets the current random_state"""
         self._validate_random_state()
@@ -1559,6 +1646,18 @@ class BaseAutoML(BaseEstimator, ABC):
             return
         check_positive_integer(self.top_models_to_improve, "top_models_to_improve")
 
+    def _validate_boost_on_errors(self):
+        """ Validates boost_on_errors parameter"""
+        if isinstance(self.boost_on_errors, str) and self.boost_on_errors == "auto":
+            return
+        check_bool(self.boost_on_errors, "boost_on_errors")
+    
+    def _validate_kmeans_features(self):
+        """ Validates kmeans_features parameter"""
+        if isinstance(self.kmeans_features, str) and self.kmeans_features == "auto":
+            return
+        check_bool(self.kmeans_features, "kmeans_features")
+
     def _validate_random_state(self):
         """ Validates random_state parameter"""
         check_positive_integer(self.random_state, "random_state")
@@ -1584,3 +1683,112 @@ class BaseAutoML(BaseEstimator, ABC):
         self._threshold = json_data.get("threshold")
 
         self._ml_task = json_data.get("ml_task")
+
+    def _md_to_html(self, md_fname, page_type):
+        import markdown
+
+        if not os.path.exists(md_fname):
+            return None
+        content = ""
+        with open(md_fname) as fin:
+            content = fin.read()
+
+        content = content.replace("README.md", "README.html")
+        content_html = markdown.markdown(
+            content, extensions=["markdown.extensions.tables"]
+        )
+        content_html = content_html.replace("<img ", '<img style="width:80%" ')
+        content_html = content_html.replace("<table>", '<table class="styled-table">')
+        content_html = content_html.replace("<tr>", '<tr style="text-align: right;">')
+
+        styles = '<link rel="stylesheet" href="style.css">\n\n'
+        if page_type == "sub":
+            styles = '<link rel="stylesheet" href="../style.css">\n\n'
+        beginning = styles
+
+        if page_type == "main":
+            beginning += """<img src="https://raw.githubusercontent.com/mljar/visual-identity/main/media/mljar_AutomatedML.png" style="height:128px; margin-left: auto;
+margin-right: auto;display: block;"/>\n\n"""
+            if os.path.exists(os.path.join(self._results_path, "EDA")):
+                beginning += '<a href="EDA/README.html">Automatic Exploratory Data Analysis Report</a>'
+
+        content_html = beginning + content_html
+
+        html_fname = md_fname.replace("README.md", "README.html")
+        with open(html_fname, "w") as fout:
+            fout.write(content_html)
+
+        return html_fname
+
+    def _report(self, width=900, height=1200):
+
+        from IPython.display import IFrame
+
+        main_readme_html = os.path.join(self._results_path, "README.html")
+        if not os.path.exists(main_readme_html) or 1:
+            fname = os.path.join(self._results_path, "README.md")
+            main_readme_html = self._md_to_html(fname, "main")
+            for f in os.listdir(self._results_path):
+                fname = os.path.join(self._results_path, f, "README.md")
+                if os.path.exists(fname):
+                    self._md_to_html(fname, "sub")
+            with open(os.path.join(self._results_path, "style.css"), "w") as fout:
+                fout.write(
+                    """
+.styled-table {
+    border-collapse: collapse;
+    font-size: 0.9em;
+    font-family:Courier New;
+}
+
+.styled-table td, .styled-table th {
+    border: 1px solid #ddd;
+    padding: 8px;
+}
+
+.styled-table tr:nth-child(even){background-color: #f2f2f2;}
+
+.styled-table tr:hover {background-color: #e0ecf5;}
+
+.styled-table th {
+    padding-top: 6px;
+    padding-bottom: 6px;
+    text-align: left;
+    background-color: #0099cc;
+    color: white;
+}
+
+body {
+    font-family: Arial;
+    background-color: rgba(236, 243, 249, 0.15);
+}
+
+h1 {
+    color: #004666;
+    border-bottom: 1px solid rgba(0,70,102,0.3)
+}
+h2 {
+    color: #004666;
+    padding-bottom: 5px;
+    margin-bottom: 0px;
+}
+
+ul {
+    margin-top: 0px;
+}
+
+p {
+    margin-top: 5px;
+}
+
+h3 {
+    color: #004666;
+    padding-bottom: 5px;
+    margin-bottom: 0px;
+}
+
+"""
+                )
+
+        if main_readme_html is not None:
+            return IFrame(main_readme_html, width, height)
