@@ -71,7 +71,15 @@ class ModelFramework:
         return self.train_time
 
     def predictions(
-        self, learner, preproces, X_train, y_train, X_validation, y_validation
+        self,
+        learner,
+        preproces,
+        X_train,
+        y_train,
+        sample_weight,
+        X_validation,
+        y_validation,
+        sample_weight_validation,
     ):
         y_train_true = y_train
         y_train_predicted = learner.predict(X_train)
@@ -95,8 +103,10 @@ class ModelFramework:
         return {
             "y_train_true": y_train_true,
             "y_train_predicted": y_train_predicted,
+            "sample_weight": sample_weight,
             "y_validation_true": y_validation_true,
             "y_validation_predicted": y_validation_predicted,
+            "sample_weight_validation": sample_weight_validation,
             "validation_index": X_validation.index,
             "validation_columns": y_validation_columns,
         }
@@ -121,15 +131,23 @@ class ModelFramework:
                         validation_data["y"].shape,
                     )
                 )
+                if "sample_weight" in train_data:
+                    logger.debug("Sample weight available during the training.")
 
                 # the proprocessing is done at every validation step
-                self.preprocessings += [Preprocessing(self.preprocessing_params)]
+                self.preprocessings += [Preprocessing(self.preprocessing_params, self.get_name(), k_fold, repeat)]
 
-                X_train, y_train = self.preprocessings[-1].fit_and_transform(
-                    train_data["X"], train_data["y"]
+                X_train, y_train, sample_weight = self.preprocessings[
+                    -1
+                ].fit_and_transform(
+                    train_data["X"], train_data["y"], train_data.get("sample_weight")
                 )
-                X_validation, y_validation = self.preprocessings[-1].transform(
-                    validation_data["X"], validation_data["y"]
+                X_validation, y_validation, sample_weight_validation = self.preprocessings[
+                    -1
+                ].transform(
+                    validation_data["X"],
+                    validation_data["y"],
+                    validation_data.get("sample_weight"),
                 )
 
                 self.learner_params["explain_level"] = self._explain_level
@@ -149,9 +167,20 @@ class ModelFramework:
                     self.callbacks.on_iteration_start()
 
                     learner.fit(
-                        X_train, y_train, X_validation, y_validation, log_to_file
+                        X_train,
+                        y_train,
+                        sample_weight,
+                        X_validation,
+                        y_validation,
+                        sample_weight_validation,
+                        log_to_file,
                     )
 
+                    if self.params.get("injected_sample_weight", False):
+                        #print("Dont use sample weight in model evaluation")
+                        sample_weight = None
+                        sample_weight_validation = None
+
                     self.callbacks.on_iteration_end(
                         {"iter_cnt": i},
                         self.predictions(
@@ -159,8 +188,10 @@ class ModelFramework:
                             self.preprocessings[-1],
                             X_train,
                             y_train,
+                            sample_weight,
                             X_validation,
                             y_validation,
+                            sample_weight_validation,
                         ),
                     )
 
@@ -195,7 +226,9 @@ class ModelFramework:
 
         # end of validation loop
         self.callbacks.on_framework_train_end()
-        self.get_additional_metrics()
+        # self.get_additional_metrics()
+        self._additional_metrics = self.get_additional_metrics()
+
         self.train_time = time.time() - start_time
         logger.debug("ModelFramework end of training")
 
@@ -216,7 +249,7 @@ class ModelFramework:
 
     def get_out_of_folds(self):
         if self.oof_predictions is not None:
-            return self.oof_predictions
+            return self.oof_predictions.copy(deep=True)
         early_stopping = self.callbacks.get("early_stopping")
         if early_stopping is None:
             return None
@@ -275,7 +308,7 @@ class ModelFramework:
         y_predicted = None  # np.zeros((X.shape[0],))
         for ind, learner in enumerate(self.learners):
             # preprocessing goes here
-            X_data, _ = self.preprocessings[ind].transform(X.copy(), None)
+            X_data, _, _ = self.preprocessings[ind].transform(X.copy(), None)
             y_p = learner.predict(X_data)
             y_p = self.preprocessings[ind].inverse_scale_target(y_p)
 
@@ -290,6 +323,7 @@ class ModelFramework:
         return y_predicted_final
 
     def get_additional_metrics(self):
+
         if self._additional_metrics is None:
             # 'target' - the target after processing used for model training
             # 'prediction' - out of folds predictions of the model
@@ -308,8 +342,12 @@ class ModelFramework:
             else:
                 oof_preds = oof_predictions[prediction_cols]
 
+            sample_weight = None
+            if "sample_weight" in oof_predictions.columns:
+                sample_weight = oof_predictions["sample_weight"]
+
             self._additional_metrics = AdditionalMetrics.compute(
-                target, oof_preds, self._ml_task
+                target, oof_preds, sample_weight, self._ml_task
             )
             if self._ml_task == BINARY_CLASSIFICATION:
                 self._threshold = float(self._additional_metrics["threshold"])
@@ -354,13 +392,18 @@ class ModelFramework:
                 desc["threshold"] = self._threshold
             fout.write(json.dumps(desc, indent=4))
 
+        learning_curve_metric = self.learners[0].get_metric_name()
+        if learning_curve_metric is None:
+            learning_curve_metric = self.get_metric_name()
+
         LearningCurves.plot(
             [l.name for l in self.learners],
-            self.get_metric_name(),
+            learning_curve_metric,
             model_path,
             trees_in_iteration=self.additional_params.get("trees_in_step"),
         )
 
+        # call additional metics just to be sure they are computed
         self._additional_metrics = self.get_additional_metrics()
 
         AdditionalMetrics.save(
@@ -378,7 +421,10 @@ class ModelFramework:
             self._ml_task, self.learner_params["model_type"]
         )
         short_name = self.learner_params["model_type"]
-        desc = f"# Summary of {self.get_name()}\n"
+        desc = f"# Summary of {self.get_name()}\n\n"
+
+        desc += "[<< Go back](../README.md)\n\n"
+
         if long_name == short_name:
             desc += f"\n## {short_name}\n"
         else:
