@@ -4,6 +4,8 @@ import json
 import numpy as np
 import pandas as pd
 
+from sklearn.preprocessing import OneHotEncoder
+
 from supervised.tuner.random_parameters import RandomParameters
 from supervised.algorithms.registry import AlgorithmsRegistry
 from supervised.preprocessing.preprocessing_categorical import PreprocessingCategorical
@@ -36,6 +38,8 @@ class MljarTuner:
         train_ensemble,
         stack_models,
         adjust_validation,
+        boost_on_errors,
+        kmeans_features,
         seed,
     ):
         logger.debug("MljarTuner.__init__")
@@ -52,6 +56,8 @@ class MljarTuner:
         self._train_ensemble = train_ensemble
         self._stack_models = stack_models
         self._adjust_validation = adjust_validation
+        self._boost_on_errors = boost_on_errors
+        self._kmeans_features = kmeans_features
         self._seed = seed
 
         self._unique_params_keys = []
@@ -83,6 +89,44 @@ class MljarTuner:
 
         return strategies
 
+    def _can_apply_kmeans_features(self):
+        if self._data_info is None:
+            return False
+
+        # are there any continous
+        continous_cols = 0
+        for k, v in self._data_info["columns_info"].items():
+            if "categorical" not in v:
+                continous_cols += 1
+        
+        # too little columns
+        if continous_cols == 0:
+            return False
+
+        # too many columns
+        if continous_cols > 300:
+            return False
+
+        # all good, can apply kmeans
+        return True
+
+    def _can_apply_golden_features(self):
+        if self._data_info is None:
+            return False
+
+        # are there any continous
+        continous_cols = 0
+        for k, v in self._data_info["columns_info"].items():
+            if "categorical" not in v:
+                continous_cols += 1
+        
+        # too little columns
+        if continous_cols == 0:
+            return False
+
+        # all good, can apply golden features
+        return True
+
     def steps(self):
 
         all_steps = []
@@ -90,6 +134,7 @@ class MljarTuner:
             all_steps += ["adjust_validation"]
 
         all_steps += ["simple_algorithms", "default_algorithms"]
+        
         if self._start_random_models > 1:
             all_steps += ["not_so_random"]
 
@@ -98,14 +143,17 @@ class MljarTuner:
             all_steps += ["mix_encoding"]
         if PreprocessingTuner.CATEGORICALS_LOO in categorical_strategies:
             all_steps += ["loo_encoding"]
-
-        if self._golden_features:
+        if self._golden_features and self._can_apply_golden_features():
             all_steps += ["golden_features"]
+        if self._kmeans_features and self._can_apply_kmeans_features():
+            all_steps += ["kmeans_features"]
         if self._features_selection:
             all_steps += ["insert_random_feature"]
             all_steps += ["features_selection"]
         for i in range(self._hill_climbing_steps):
             all_steps += [f"hill_climbing_{i+1}"]
+        if self._boost_on_errors:
+            all_steps += ["boost_on_errors"]
         if self._train_ensemble:
             all_steps += ["ensemble"]
         if self._stack_models:
@@ -141,6 +189,10 @@ class MljarTuner:
                 return self.get_golden_features_params(
                     models, results_path, total_time_limit
                 )
+            elif step == "kmeans_features":
+                return self.get_kmeans_features_params(
+                    models, results_path, total_time_limit
+                )
             elif step == "insert_random_feature":
                 return self.get_params_to_insert_random_feature(
                     models, total_time_limit
@@ -155,6 +207,8 @@ class MljarTuner:
                 return self.get_hill_climbing_params(
                     self.filter_random_feature_model(models)
                 )
+            elif step == "boost_on_errors":
+                return self.boost_params(models, results_path)
             elif step == "ensemble":
                 return [
                     {
@@ -204,11 +258,15 @@ class MljarTuner:
         generated_params = []
         # resue old params
         for m in stacked_models:
-            # print(m.get_type())
             # use only Xgboost, LightGBM and CatBoost as stacked models
             if m.get_type() not in ["Xgboost", "LightGBM", "CatBoost"]:
                 continue
 
+            if m.params.get("injected_sample_weight", False):
+                # dont use boost_on_errors model for stacking
+                # there will be additional boost_on_errors step
+                continue
+
             params = copy.deepcopy(m.params)
 
             params["validation_strategy"]["X_path"] = params["validation_strategy"][
@@ -217,7 +275,6 @@ class MljarTuner:
 
             params["name"] = params["name"] + "_Stacked"
             params["is_stacked"] = True
-            # print(params)
             params["status"] = "initialized"
             params["final_loss"] = None
             params["train_time"] = None
@@ -260,7 +317,6 @@ class MljarTuner:
             unique_params_key = MljarTuner.get_params_key(params)
             if unique_params_key not in self._unique_params_keys:
                 generated_params += [params]
-                self._unique_params_keys += [unique_params_key]
                 models_cnt += 1
         return generated_params
 
@@ -286,7 +342,6 @@ class MljarTuner:
                 unique_params_key = MljarTuner.get_params_key(params)
                 if unique_params_key not in self._unique_params_keys:
                     generated_params += [params]
-                    self._unique_params_keys += [unique_params_key]
                     models_cnt += 1
         return generated_params
 
@@ -312,8 +367,8 @@ class MljarTuner:
 
         generated_params = []
         for model_type in [
-            "Xgboost",
             "LightGBM",
+            "Xgboost",
             "CatBoost",
             "Neural Network",
             "Random Forest",
@@ -342,7 +397,6 @@ class MljarTuner:
             unique_params_key = MljarTuner.get_params_key(params)
             if unique_params_key not in self._unique_params_keys:
                 generated_params += [params]
-                self._unique_params_keys += [unique_params_key]
                 models_cnt += 1
         return generated_params
 
@@ -384,9 +438,8 @@ class MljarTuner:
                 unique_params_key = MljarTuner.get_params_key(params)
                 if unique_params_key not in self._unique_params_keys:
                     generated_params[model_type] += [params]
-                    self._unique_params_keys += [unique_params_key]
                     models_cnt += 1
-
+        
         """
         return_params = []
         for i in range(100):
@@ -414,8 +467,8 @@ class MljarTuner:
         for i in range(100):
             total = 0
             for m in [
-                "Xgboost",
                 "LightGBM",
+                "Xgboost",
                 "CatBoost",
                 "Random Forest",
                 "Extra Trees",
@@ -475,9 +528,10 @@ class MljarTuner:
                     all_params["final_loss"] = None
                     all_params["train_time"] = None
                     unique_params_key = MljarTuner.get_params_key(all_params)
+                    
                     if unique_params_key not in self._unique_params_keys:
-                        self._unique_params_keys += [unique_params_key]
                         generated_params += [all_params]
+                    
         return generated_params
 
     def get_all_int_categorical_strategy(self, current_models, total_time_limit):
@@ -498,7 +552,7 @@ class MljarTuner:
     def get_categorical_strategy(self, current_models, strategy, total_time_limit):
 
         df_models, algorithms = self.df_models_algorithms(
-            current_models, time_limit=0.03 * total_time_limit
+            current_models, time_limit=0.1 * total_time_limit
         )
         generated_params = []
         for m_type in algorithms:
@@ -570,11 +624,10 @@ class MljarTuner:
                     del params["learner"]["model_architecture_json"]
                 unique_params_key = MljarTuner.get_params_key(params)
                 if unique_params_key not in self._unique_params_keys:
-                    self._unique_params_keys += [unique_params_key]
                     generated_params += [params]
         return generated_params
 
-    def df_models_algorithms(self, current_models, time_limit=None):
+    def df_models_algorithms(self, current_models, time_limit=None, exclude_golden=False):
         scores = [m.get_final_loss() for m in current_models]
         model_types = [m.get_type() for m in current_models]
         names = [m.get_name() for m in current_models]
@@ -593,13 +646,16 @@ class MljarTuner:
             df_models = df_models[df_models.train_time < time_limit]
             df_models.reset_index(drop=True, inplace=True)
 
+        if exclude_golden:
+            ii = df_models["name"].apply(lambda x: "GoldenFeatures" in x)
+            df_models = df_models[~ii]
+            df_models.reset_index(drop=True, inplace=True)
+
         df_models.sort_values(by="score", ascending=True, inplace=True)
         model_types = list(df_models.model_type)
         u, idx = np.unique(model_types, return_index=True)
         algorithms = u[np.argsort(idx)]
 
-        # print(df_models)
-        # print(algorithms)
         return df_models, algorithms
 
     def get_golden_features_params(
@@ -607,41 +663,61 @@ class MljarTuner:
     ):
 
         df_models, algorithms = self.df_models_algorithms(
-            current_models, time_limit=0.03 * total_time_limit
+            current_models, time_limit=0.1 * total_time_limit
         )
 
         generated_params = []
-        for m_type in algorithms:
-            # try to add golden features only for below algorithms
-            if m_type not in ["Xgboost", "LightGBM", "CatBoost"]:
-                continue
-            models = df_models[df_models.model_type == m_type]["model"]
+        for i in range(min(3, df_models.shape[0])):
+            m = df_models["model"].iloc[i]
 
-            for i in range(min(1, len(models))):
-                m = models.iloc[i]
+            params = copy.deepcopy(m.params)
+            params["preprocessing"]["golden_features"] = {
+                "results_path": results_path,
+                "ml_task": self._ml_task,
+            }
+            params["name"] += "_GoldenFeatures"
+            params["status"] = "initialized"
+            params["final_loss"] = None
+            params["train_time"] = None
 
-                params = copy.deepcopy(m.params)
-                params["preprocessing"]["golden_features"] = {
-                    "results_path": results_path,
-                    "ml_task": self._ml_task,
-                }
-                params["name"] += "_GoldenFeatures"
-                params["status"] = "initialized"
-                params["final_loss"] = None
-                params["train_time"] = None
+            if "model_architecture_json" in params["learner"]:
+                del params["learner"]["model_architecture_json"]
+            unique_params_key = MljarTuner.get_params_key(params)
+            if unique_params_key not in self._unique_params_keys:
+                generated_params += [params]
+        return generated_params
 
-                if "model_architecture_json" in params["learner"]:
-                    del params["learner"]["model_architecture_json"]
-                unique_params_key = MljarTuner.get_params_key(params)
-                if unique_params_key not in self._unique_params_keys:
-                    self._unique_params_keys += [unique_params_key]
-                    generated_params += [params]
+    def get_kmeans_features_params(
+        self, current_models, results_path, total_time_limit
+    ):
+
+        df_models, algorithms = self.df_models_algorithms(
+            current_models, time_limit=0.1 * total_time_limit,
+            exclude_golden=True
+        )
+
+        generated_params = []
+        for i in range(min(3, df_models.shape[0])):
+            m = df_models["model"].iloc[i]
+
+            params = copy.deepcopy(m.params)
+            params["preprocessing"]["kmeans_features"] = {"results_path": results_path}
+            params["name"] += "_KMeansFeatures"
+            params["status"] = "initialized"
+            params["final_loss"] = None
+            params["train_time"] = None
+
+            if "model_architecture_json" in params["learner"]:
+                del params["learner"]["model_architecture_json"]
+            unique_params_key = MljarTuner.get_params_key(params)
+            if unique_params_key not in self._unique_params_keys:
+                generated_params += [params]
         return generated_params
 
     def time_features_selection(self, current_models, total_time_limit):
 
         df_models, algorithms = self.df_models_algorithms(
-            current_models, time_limit=0.05 * total_time_limit
+            current_models, time_limit=0.1 * total_time_limit
         )
 
         time_needed = 0
@@ -667,7 +743,7 @@ class MljarTuner:
                     time_needed += 2.0 * m.get_train_time()
                 else:
                     time_needed += m.get_train_time()
-                # print(m.get_name(), m.get_final_loss(), time_needed)
+                
 
         return time_needed
 
@@ -686,7 +762,7 @@ class MljarTuner:
             return None
 
         df_models, algorithms = self.df_models_algorithms(
-            current_models, time_limit=0.05 * total_time_limit
+            current_models, time_limit=0.1 * total_time_limit
         )
         if df_models.shape[0] == 0:
             return None
@@ -705,7 +781,6 @@ class MljarTuner:
 
         unique_params_key = MljarTuner.get_params_key(params)
         if unique_params_key not in self._unique_params_keys:
-            self._unique_params_keys += [unique_params_key]
             return [params]
         return None
 
@@ -715,7 +790,6 @@ class MljarTuner:
 
         fname = os.path.join(results_path, "drop_features.json")
         if not os.path.exists(fname):
-            # print("The file with features to drop is missing")
             return None
 
         drop_features = json.load(open(fname, "r"))
@@ -727,7 +801,7 @@ class MljarTuner:
             return None
 
         df_models, algorithms = self.df_models_algorithms(
-            current_models, time_limit=0.05 * total_time_limit
+            current_models, time_limit=0.1 * total_time_limit
         )
 
         generated_params = []
@@ -757,7 +831,6 @@ class MljarTuner:
                     del params["learner"]["model_architecture_json"]
                 unique_params_key = MljarTuner.get_params_key(params)
                 if unique_params_key not in self._unique_params_keys:
-                    self._unique_params_keys += [unique_params_key]
                     generated_params += [params]
         return generated_params
 
@@ -803,10 +876,82 @@ class MljarTuner:
     @staticmethod
     def get_params_key(params):
         key = "key_"
-        for main_key in ["preprocessing", "learner"]:
-            key += main_key
-            for k, v in params[main_key].items():
-                if k == "seed":
+        for main_key in ["preprocessing", "learner", "validation_strategy"]:
+            key += "_" + main_key
+            for k in sorted(params[main_key]):
+                if k in ["seed", "explain_level"]:
                     continue
-                key += "_{}_{}".format(k, v)
+                key += "_{}_{}".format(k, params[main_key][k])
         return key
+
+    def add_key(self, model):
+        if model.get_type() != "Ensemble":
+            key = MljarTuner.get_params_key(model.params)
+            self._unique_params_keys += [key]
+
+
+    def boost_params(self, current_models, results_path):
+
+        df_models, algorithms = self.df_models_algorithms(current_models)
+        best_model = None
+        for i in range(df_models.shape[0]):
+            if df_models["model_type"].iloc[i] in [
+                "Ensemble",
+                "Neural Network",
+                "Nearest Neighbors",
+            ]:
+                continue
+            best_model = df_models["model"].iloc[i]
+            break
+        if best_model is None:
+            return []
+
+        # load predictions
+        oof = best_model.get_out_of_folds()
+
+        predictions = oof[[c for c in oof.columns if c.startswith("prediction")]]
+        y = oof["target"]
+
+        if self._ml_task == MULTICLASS_CLASSIFICATION:
+            oh = OneHotEncoder(sparse=False)
+            y_encoded = oh.fit_transform(np.array(y).reshape(-1, 1))
+            residua = np.sum(
+                np.abs(np.array(y_encoded) - np.array(predictions)), axis=1
+            )
+        else:
+            residua = np.abs(np.array(y) - np.array(predictions).ravel())
+
+        df_preds = pd.DataFrame(
+            {"res": residua, "lp": range(residua.shape[0]), "target": np.array(y)}
+        )
+
+        df_preds = df_preds.sort_values(by="res", ascending=True)
+        df_preds["order"] = range(residua.shape[0])
+        df_preds["order"] = (df_preds["order"]) / residua.shape[0] / 5.0 + 0.9
+        df_preds = df_preds.sort_values(by="lp", ascending=True)
+
+        sample_weight_path = os.path.join(
+            results_path, best_model.get_name() + "_sample_weight.parquet"
+        )
+        pd.DataFrame({"sample_weight": df_preds["order"]}).to_parquet(
+            sample_weight_path, index=False
+        )
+
+        generated_params = []
+
+        params = copy.deepcopy(best_model.params)
+        
+        params["validation_strategy"]["sample_weight_path"] = sample_weight_path
+        params["injected_sample_weight"] = True
+        params["name"] += "_BoostOnErrors"
+        params["status"] = "initialized"
+        params["final_loss"] = None
+        params["train_time"] = None
+        if "model_architecture_json" in params["learner"]:
+            del params["learner"]["model_architecture_json"]
+        unique_params_key = MljarTuner.get_params_key(params)
+
+        if unique_params_key not in self._unique_params_keys:
+            generated_params += [params]
+
+        return generated_params
